use crate::host::{Host, ParseHostError};
use std::{fmt, str::FromStr};

#[derive(Clone, Debug, PartialEq)]
pub struct Request {
    user: String,
    host: Host,
    message: String,
}

impl fmt::Display for Request {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "misfin://{}@{} {}\r\n", self.user, self.host, self.message)
    }
}

#[derive(Clone, Debug, PartialEq)]
pub enum ParseRequestError {
    MissingSeparator,
    EmptyUser,
    EmptyHost,
    EmptyMessage,
    Malformed,
    ParseHostError(ParseHostError),
}

impl fmt::Display for ParseRequestError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::ParseHostError(e) => write!(f, "{e}"),
            _ => write!(f, "{self:?}")
        }
    }
}

impl std::error::Error for ParseRequestError {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::ParseHostError(e) => Some(e),
            _ => None,
        }
    }
}

impl From<ParseHostError> for ParseRequestError {
    fn from(value: ParseHostError) -> Self {
        Self::ParseHostError(value)
    }
}

impl FromStr for Request {
    type Err = ParseRequestError;

    fn from_str(s: &str) -> Result<Self, Self::Err> {
        if !s.ends_with("\r\n") {
            return Err(ParseRequestError::Malformed);
        }
        if let Some((user, message)) = s.split_once(' ') {
            let message = message.trim_end().to_string();
            if let Some((user, host)) = user.rsplit_once('@') {
                let host = host.parse()?;
                let user = user.to_string();
                Ok(Request { user, host, message })
            } else {
                Err(ParseRequestError::MissingSeparator)
            }
        } else {
            Err(ParseRequestError::MissingSeparator)
        }
    }
}
